import os
import click
import subprocess
import webbrowser
import socket
from contextlib import closing
from .TemporaryDirectory import TemporaryDirectory


@click.group()
def neurosift():
    pass


@click.command()
@click.argument('file', type=click.Path(exists=True))
def view_nwb(file):
    abs_fname = os.path.abspath(file)
    base_fname = os.path.basename(abs_fname)
    with TemporaryDirectory(prefix="view_nwb") as tmpdir:
        # create a symbolic link to the file
        os.symlink(abs_fname, f'{tmpdir}/{base_fname}')

        # this directory
        this_directory = os.path.dirname(os.path.realpath(__file__))

        # apparently shell=True is necessary for Windows, but shell=False is necessary for Linux
        if os.name == 'nt':
            shell = True
        elif os.name == 'posix':
            shell = False
        else:
            print(f'Warning: unrecognized os.name: {os.name}')
            shell = False

        try:
            npm_version = subprocess.run(["npm", "--version"], stdout=subprocess.PIPE, universal_newlines=True, shell=shell).stdout.strip()
            print(f'npm version: {npm_version}')
        except Exception:
            raise Exception('Unable to run npm.')

        try:
            node_version = subprocess.run(["node", "--version"], stdout=subprocess.PIPE, universal_newlines=True, shell=shell).stdout.strip()
            print(f'node version: {node_version}')
        except Exception:
            raise Exception('Unable to run node.')

        # parse node_version v18.0.0 to get the major version number
        node_major_version = int(node_version.split('.')[0][1:])
        if node_major_version < 16:
            raise Exception('node version must be >= 16.0.0')

        # run the command npm install in the js directory
        subprocess.run(["npm", "install"], cwd=f'{this_directory}/local-file-access-js', shell=shell)

        # find an open port
        port = find_free_port()

        # run the service
        process = subprocess.Popen(['npm', 'run', 'start', tmpdir], cwd=f'{this_directory}/local-file-access-js', shell=shell, env=dict(os.environ, PORT=str(port)))

        # open the browser
        url = f"https://flatironinstitute.github.io/neurosift/?p=/nwb&url=http://localhost:{port}/files/{base_fname}"
        print(f'Opening {url}')
        webbrowser.open(url)

        # wait for the process to finish
        process.wait()


def find_free_port():
    with closing(socket.socket(socket.AF_INET, socket.SOCK_STREAM)) as s:
        s.bind(('', 0))
        s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        return s.getsockname()[1]


# Add command to the neurosift group
neurosift.add_command(view_nwb)


if __name__ == '__main__':
    neurosift()



import os
import shutil
import time
import random
import tempfile

class TemporaryDirectory():
    def __init__(self, *, remove: bool=True, prefix: str='tmp'):
        self._remove = remove
        self._prefix = prefix

    def __enter__(self) -> str:
        tmpdir = tempfile.gettempdir()
        self._path = f'{tmpdir}/{self._prefix}_{_random_string(8)}'
        os.mkdir(self._path)
        return self._path

    def __exit__(self, exc_type, exc_val, exc_tb):
        if self._remove:
            if not os.getenv('KACHERY_CLOUD_KEEP_TEMP_FILES') == '1':
                _rmdir_with_retries(self._path, num_retries=5)

    def path(self):
        return self._path


def _rmdir_with_retries(dirname: str, num_retries: int, delay_between_tries: float=1):
    for retry_num in range(1, num_retries + 1):
        if not os.path.exists(dirname):
            return
        try:
            shutil.rmtree(dirname)
            break
        except: # pragma: no cover
            if retry_num < num_retries:
                print('Retrying to remove directory: {}'.format(dirname))
                time.sleep(delay_between_tries)
            else:
                raise Exception('Unable to remove directory after {} tries: {}'.format(num_retries, dirname))

def _random_string(num_chars: int) -> str:
    chars = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789'
    return ''.join(random.choice(chars) for _ in range(num_chars))


from setuptools import setup, find_packages

with open('README.md', 'r', encoding='utf-8') as f:
    long_description = f.read()

setup(
    packages=find_packages(),
    scripts=[],
    include_package_data=True,
    package_data={},
    install_requires=[
        'click'
    ],
    long_description=long_description,
    long_description_content_type='text/markdown',  # This is important!
    entry_points={
        "console_scripts": [
            "neurosift=neurosift.cli:neurosift",
        ],
    }
)
